import React, { Component, Fragment } from 'react'
import PropTypes from 'prop-types'
import OAC from 'oneaccount-js-core'
import './styles.css'

// TODO: make easier to change styles
// TODO: move styles.css to oneaccount.css in library
// TODO: make use of children property of react to inject html

const TitleSVG = ({ color }) => (
  <svg fill={color} viewBox="0 0 320 66" xmlns="http://www.w3.org/2000/svg">
    <text textAnchor="middle" x="50%" y="20">
      Download One account mobile
    </text>
    <text textAnchor="middle" x="50%" y="43">
      app and scan the QR code
    </text>
    <text textAnchor="middle" fontSize="10px" x="50%" y="63">
      or click the QR code if One account is installed on current device
    </text>
  </svg>
)

// const IS_MOBILE = /iPhone|Android/i.test(navigator.userAgent)
const isMobile = a => {
  if (
    /(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino/i.test(
      a
    ) ||
    /1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|\-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\-(n|u)|c55\/|capi|ccwa|cdm\-|cell|chtm|cldc|cmd\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc\-s|devi|dica|dmob|do(c|p)o|ds(12|\-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\-|_)|g1 u|g560|gene|gf\-5|g\-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd\-(m|p|t)|hei\-|hi(pt|ta)|hp( i|ip)|hs\-c|ht(c(\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i\-(20|go|ma)|i230|iac( |\-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc\-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|\-[a-w])|libw|lynx|m1\-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m\-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\-2|po(ck|rt|se)|prox|psio|pt\-g|qa\-a|qc(07|12|21|32|60|\-[2-7]|i\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h\-|oo|p\-)|sdk\/|se(c(\-|0|1)|47|mc|nd|ri)|sgh\-|shar|sie(\-|m)|sk\-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h\-|v\-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl\-|tdg\-|tel(i|m)|tim\-|t\-mo|to(pl|sh)|ts(70|m\-|m3|m5)|tx\-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|\-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(\-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas\-|your|zeto|zte\-/i.test(
      a.substr(0, 4)
    )
  ) {
    return true
  }
  return false
}

const IS_MOBILE = isMobile(
  navigator.userAgent || navigator.vendor || window.opera
)
const ANIMATION_DURATION = 3000

class OneAccount extends Component {
  constructor(props) {
    super(props)
    this.container = React.createRef()
    this.floatingModeButton = React.createRef()
    this.updateQRCode = this.updateQRCode.bind(this)

    this.toVDOM = this.toVDOM.bind(this)
    this.toggle = this.toggle.bind(this)
    const { floatingMode } = this.props
    this.state = {
      isOpen: !floatingMode,
      showSpinner: false,
      running: false
    }
    this.base = 'oneaccount-react'
    if (this.props.updateInterval < 0) this.props.updateInterval = 0

    this.oac = new OAC(this.props.externalId, {
      color: this.props.color,
      level: this.props.level,
      domain: this.props.domain
    })

    this.qrcode = this.oac.QRCode()
    this.props.onUpdate(this.oac.UUID())
  }

  refreshOAC() {
    this.qrcode = this.oac.QRCode()
    this.props.onUpdate(this.oac.UUID())
  }

  hideOnClickOutside(element) {
    const outsideClickListener = event => {
      if (
        !this.container.current.contains(event.target) &&
        !this.floatingModeButton.current.contains(event.target)
      ) {
        // or use: event.target.closest(selector) === null
        this.stop()
        this.setState({ isOpen: false })
        removeClickListener()
      }
    }
    // TODO: remove listener when clicked on floatingButton
    const removeClickListener = () => {
      document.removeEventListener('click', outsideClickListener)
    }

    document.addEventListener('click', outsideClickListener)
  }

  componentDidMount() {
    // NOTE: maybe add this to head?
    let style = document.createElement('style')
    style.innerHTML =
      this.oac.GlobalCSS() +
      `.oneaccount-mask-group {
      -webkit-animation: raise ${
      this.props.updateInterval
      }ms infinite ease-in-out !important;
      animation: raise ${
      this.props.updateInterval
      }ms infinite ease-in-out !important;
    }`
    this.container.current.appendChild(style)
    this.start()
  }

  start() {
    if (this.state.running) return
    this.setState({ running: true })
    this.timerQRCode = setInterval(
      () => this.updateQRCode(),
      this.props.updateInterval
    )
  }

  componentWillUnmount() {
    this.stop()
  }
  // componentWillUpdate(nextProps, nextState) {
  //   console.log(1, nextProps, nextState);
  // }

  updateQRCode() {
    let count = ANIMATION_DURATION / 1000
    this.setState({ showSpinner: true })
    this.timerQRCodeUpdate = setInterval(() => {
      if (count <= 1) {
        this.stop()
        this.start()
        this.setState({ showSpinner: false })
        clearInterval(this.timerQRCodeUpdate)
      }
      count--
    }, 1000)
  }

  stop() {
    this.refreshOAC()
    this.setState({ showSpinner: false })
    this.setState({ running: false })
    clearInterval(this.timerQRCode)
    clearInterval(this.timerQRCodeUpdate)
  }

  toggle() {
    let a = document.createElement('a')
    if (!this.props.floatingMode) return
    if (IS_MOBILE) {
      window.location = this.oac.URL()
      return
    }
    if (!this.state.isOpen) {
      this.start()
      this.hideOnClickOutside()
    } else {
      this.stop()
    }
    this.setState({ isOpen: !this.state.isOpen })
  }

  toVDOM({ svg, onClick, className = '' } = {}) {
    return (
      <svg
        className={svg.className + ' ' + className}
        viewBox={svg.viewBox}
        onClick={onClick}
      >
        {svg.paths.map((obj, index) => {
          if (obj.type === 'circle') {
            return (
              <circle
                key={index}
                stroke={obj.stroke}
                strokeWidth={obj.strokeWidth}
                fill={obj.fill}
                cx={obj.cx}
                cy={obj.cy}
                r={obj.r}
              />
            )
          } else if (obj.type === 'mask') {
            return (
              <mask key={index} id={obj.id}>
                <g fill="white" className="oneaccount-mask-group">
                  <path fill={obj.fill} d={obj.path} />
                </g>
              </mask>
            )
          } else {
            return (
              <path mask={obj.mask} key={index} fill={obj.fill} d={obj.path} />
            )
          }
        })}
      </svg>
    )
  }

  render() {
    const { floatingMode, title, authText, icons } = this.props
    const color = this.props.color || '#3C444F'
    let { isOpen, showSpinner, running } = this.state
    const logo = this.toVDOM({
      svg: this.oac.LoadingLogo(
        undefined,
        'oneaccount-logo oneaccount-loading-logo'
      )
    })
    const cross = this.toVDOM({ svg: this.oac.IconCross('white') })
    const roundLogo = this.toVDOM({
      svg: this.oac.Logo(undefined, 'oneaccount-logo-round')
    })
    const spinner = this.toVDOM({
      svg: this.oac.Spinner(),
      className: showSpinner ? 'show-spinner' : ''
    })
    const qrcode = this.toVDOM({ svg: this.qrcode })
    const iconAndroid = this.toVDOM({ svg: this.oac.IconAndroid() })
    const iconIOS = this.toVDOM({ svg: this.oac.IconIOS() })

    let containerClass = `${floatingMode ? `${this.base}-floating-mode ` : ''}`
    if (floatingMode && isOpen) {
      containerClass += `${this.base}-show-qrcode`
    }

    // open mac app
    // TODO: handle this correctly!
    // const link = `oneaccount://oneaccount.app?externalId=${
    //   this.props.externalId
    // }&uuid=${this.oac.UUID()}&expires=${this.oac.Expires()}`

    return (
      <div className="oneaccount">
        {floatingMode && (
          <div
            className={`${this.base}-logo-wrapper`}
            onClick={this.toggle}
            ref={this.floatingModeButton}
          >
            {authText && (
              <span className={!isOpen ? 'auth-text' : 'auth-text-hidden'}>
                {authText !== true ? authText : 'Sign in/up'}
              </span>
            )}
            <div
              style={{ background: color }}
              className={`${this.base}-logo-button ${this.base}-logo-button${
                isOpen ? '-close' : '-open'
                }`}
            >
              {!isOpen ? floatingMode.logo || roundLogo : cross}
            </div>
          </div>
        )}
        {/* TODO: move these styles to css*/}
        {/* maybe refactor this to use portal using oneaccount? */}
        {/* https://reactjs.org/docs/portals.html*/}
        {/* https://reactjs.org/blog/2017/11/28/react-v16.2.0-fragment-support.html */}
        <div ref={this.container} className={containerClass}>
          {this.props.children || (
            <Fragment>
              {title && (
                <p className="oneaccount-title">
                  {title !== true ? title : <TitleSVG color={color} />}
                </p>
              )}
              {icons &&
                (icons !== true ? (
                  icons
                ) : (
                    <Fragment>
                      <a href="https://apps.apple.com/de/app/one-account/id1428861716?l=en">{iconIOS}</a>
                      <a className="unavailable" href="#">
                        {iconAndroid}
                      </a>
                    </Fragment>
                  ))}
            </Fragment>
          )}
          <div
            className="oneaccount-container"
            onClick={() => {
              if (!showSpinner) {
                window.location = this.oac.URL()
              }
            }}
          >
            {running && (this.props.logo || (showSpinner ? '' : logo))}
            {spinner}
            {qrcode}
          </div>
        </div>
      </div>
    )
  }
}

OneAccount.propTypes = {
  externalId: PropTypes.string.isRequired,
  updateInterval: PropTypes.number,
  level: PropTypes.string,
  color: PropTypes.string,
  // NOTE: maybe temp
  domain: PropTypes.string,
  logo: PropTypes.element,
  floatingMode: PropTypes.oneOfType([
    PropTypes.shape({
      logo: PropTypes.element
    }),
    PropTypes.bool
  ]),
  title: PropTypes.oneOfType([
    PropTypes.string,
    PropTypes.element,
    PropTypes.bool
  ]),
  authText: PropTypes.oneOfType([
    PropTypes.string,
    PropTypes.element,
    PropTypes.bool
  ]),
  icons: PropTypes.oneOfType([PropTypes.bool, PropTypes.element]),
  onUpdate: PropTypes.func
}

OneAccount.defaultProps = {
  ...Component.defaultProps,
  logo: null,
  floatingMode: false,
  updateInterval: 3 * 60 * 1000,
  level: 'M',
  color: '#3C444F',
  onUpdate: () => { }
}

export default OneAccount
